/*****************************************************************************
 * cmem.c
 *
 * bc-cat module unit test - BC_MEMORY_USERPTR feature
 * use cmem buffer to store YUV frames, and pass them to bc_cat.ko
 * run loadmodules.sh before run the app
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *   
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the
 *   distribution.
 *   
 *   Neither the name of Texas Instruments Incorporated nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/
#include <stdio.h>
#include <stdlib.h>     /*malloc() & free()*/
#include <img_defs.h>
#include <servicesext.h>
#include <bc_cat.h>
#include <cmem.h>

#define FRAME_WIDTH     128
#define FRAME_HEIGHT    128
#define FRAME_SIZE      (FRAME_WIDTH * FRAME_HEIGHT * 2)
#define MAX_FRAMES      1
#define YUV_PIXEL_FMT   PVRSRV_PIXEL_FORMAT_FOURCC_ORG_UYVY

typedef struct buf_info {
    char *start;
    unsigned long pa;
} buf_info_t;

void pattern_uyvy(void *p, int w, int h);
void frame_cleanup(void);

static buf_info_t frame[MAX_FRAMES];
static char *yuv_data = NULL;
static unsigned long yuv_phys = 0;
static int   fr_idx = 0;
static CMEM_AllocParams params = { CMEM_POOL, CMEM_NONCACHED, 4096 };

int frame_init(bc_buf_params_t *p)
{
    int   ii;

    if (CMEM_init() == -1) {
        printf("CMEM_init() failed\n");
        return -1;
    }

    yuv_data = CMEM_alloc(FRAME_SIZE * MAX_FRAMES, &params);
    if (yuv_data == NULL) {
        fprintf(stdout, "CMEM_alloc() failed\n");
        CMEM_exit();
        return -1;
    }

    yuv_phys = CMEM_getPhys(yuv_data);
    if (!yuv_phys) {
        fprintf(stdout, "CMEM_getPhys() failed\n");
        frame_cleanup();
        return -1;
    }

    for (ii = 0; ii < MAX_FRAMES; ii++) {
        frame[ii].start = &yuv_data[ii * FRAME_SIZE];
        frame[ii].pa = yuv_phys + ii * FRAME_SIZE;
    }

    if (p) {
        p->count = MAX_FRAMES;
        p->width = FRAME_WIDTH;
        p->height = FRAME_HEIGHT;
        p->pixel_fmt = YUV_PIXEL_FMT;
        p->type = BC_MEMORY_USERPTR;      
    }

    return 0;
}

char *frame_get(bc_buf_ptr_t *p)
{
    char *va;
    va = frame[fr_idx].start;
    pattern_uyvy(va, FRAME_WIDTH, FRAME_HEIGHT);

    if (p) {
        p->index = fr_idx;
        p->pa = frame[fr_idx].pa;
    }

    fr_idx = (fr_idx + 1) % MAX_FRAMES;
    return va;
}

int frame_restore(char *p)
{
    p = p;
    return 0;
}

void frame_cleanup(void)
{
    if (yuv_data)
        CMEM_free(yuv_data, &params);

    CMEM_exit();
}
