/*****************************************************************************
 * pattern.c
 *
 * bc-cat module unit test - create texture pattern
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *   
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the
 *   distribution.
 *   
 *   Neither the name of Texas Instruments Incorporated nor the names of
 *   its contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/
#define BARS     8  /*number of color bars*/
#define STEPS   16  /*moving steps of a bar*/

int colors[BARS] = {
           /* y       |   v       |   y      | u */
            128 << 24 | 128 << 16 | 128 << 8 | 128, /* white  */
            162 << 24 | 142 << 16 | 162 << 8 |  44, /* yellow */
            131 << 24 |  44 << 16 | 131 << 8 | 156, /* cyan   */
            112 << 24 |  58 << 16 | 112 << 8 |  72, /* green  */
             84 << 24 | 198 << 16 |  84 << 8 | 184, /* mag    */
             65 << 24 | 212 << 16 |  65 << 8 | 100, /* red    */
             35 << 24 | 114 << 16 |  35 << 8 | 212, /* blue   */
             16 << 24 | 128 << 16 |  16 << 8 | 128  /* black  */

};

void pattern_uyvy(void *p, int w, int h)
{
    static int moves = 0;
	unsigned int *t = (unsigned int *)p;
	int ii, jj, kk, c0, c;

    /*first partial strip*/
    c0 = colors[(moves/STEPS)%BARS];
    for (jj = 0; jj < h/BARS/STEPS * (STEPS - moves%STEPS); jj++) {
        for (ii = 0; ii < w; ii += 2)
            *t++ = c0;
    }

    /*all middle complete strips*/
    for (kk = 1; kk < BARS; kk++) {
        c = colors[(kk + moves/STEPS)%BARS];
        for (jj = 0; jj < h/BARS; jj++) {
            for (ii = 0; ii < w; ii += 2)
                *t++ = c;
        }
    }

    /*the remder partial strip*/
    for (jj = 0; jj < h/BARS/STEPS * (moves%STEPS); jj++) {
        for (ii = 0; ii < w; ii += 2)
            *t++ = c0;
    }
    moves++;
}
